@@ -20,7 +20,7 @@ import astropy.wcs as pywcs
 import astropy.constants as const
 
 from . import psutil
-from . import get_version
+from . import __version__
 
 
 def _fmhz(freqs, precision=3):
@@ -438,7 +438,7 @@ class ImageMetaData(object):
         header.update(self.kargs)
 
         if add_origin:
-            header['ORIGIN'] = 'ps_eor v_%s' % get_version()
+            header['ORIGIN'] = 'ps_eor v_%s' % __version__
 
         return header
 
@@ -1089,7 +1089,7 @@ class CartDataCube(DataCube):
             return ft_I_cube
 
         b_theta_fov = None
-        if trim_method == 'before':
+        if trim_method in ['b', 'before']:
             b_theta_fov = theta_fov
 
         ft_I_cube = CartDataCube.load_from_fits_image(files, umin, umax, b_theta_fov, imager_scale_factor=1,
@@ -1102,7 +1102,8 @@ class CartDataCube(DataCube):
                                                                      window_function=window_function)
 
         f = 1 / float(ft_I_cube.meta.shape[0] ** 2)
-        d_over_psf = np.where(ft_psf_cube.data != 0, np.divide(ft_I_cube.data, ft_psf_cube.data), 0)
+        with np.errstate(divide='ignore', invalid='ignore'):
+            d_over_psf = np.where(ft_psf_cube.data != 0, np.divide(ft_I_cube.data, ft_psf_cube.data), 0)
         ft_I_rw_cube = ft_I_cube.new_with_data(d_over_psf * f, weights=weight_cube)
 
         if min_weight_ratio > 0:
@@ -1110,7 +1111,7 @@ class CartDataCube(DataCube):
             min_weight = np.max([abs_min_weight, min_weight])
             ft_I_rw_cube.filter_min_weight(min_weight, replace=False)
 
-        if trim_method == 'after':
+        if trim_method in ['after', 'a']:
             ft_I_rw_cube = ft_I_rw_cube.reduce_fov(theta_fov, umin=umin, umax=umax)
 
         return ft_I_rw_cube
@@ -1830,13 +1831,16 @@ class CartWeightCube(CartDataCube):
 
     def simulate_noise(self, sefd, time, hermitian=True, weights_uncertainity_ratio=None,
                        sefd_jansky=True, fake_apply_win_fct=False):
-        df = psutil.robust_freq_width(self.freqs)
+        df = self.meta.chan_width
         int_time = self.meta.int_time
         total_time = self.meta.total_time
         f = time / total_time
 
         noise_rms = sefd / (2 * df * int_time * f) ** 0.5
 
+        # noise_rms for each complex part
+        noise_rms = np.sqrt(0.5) * noise_rms
+
         if fake_apply_win_fct:
             # Scale noise_rms as if a spatial tapering had been applied.
             noise_rms = noise_rms * self.meta.win_fct_power ** .5
