@@ -7,12 +7,12 @@ from __future__ import print_function
 from __future__ import absolute_import
 
 import os
+import time
 from io import open
 
 import json
 
 import numpy as np
-from scipy.optimize import minimize_scalar, brute, fmin
 
 from . import fitutil
 from . import datacube
@@ -42,7 +42,7 @@ class GprForegroundResult(FitterResult):
 
     def __init__(self, gpr_res, cube_fit, cube_sub, cube_pre_fit,
                  cube_post_fit, fg_med=None, eor=None, fg_med_full=None,
-                 int_fg_full=None, kern=None):
+                 int_fg_full=None, kern=None, cube_noise=None):
         """Encapsulate GPR foreground fitting result
 
         Args:
@@ -57,6 +57,7 @@ class GprForegroundResult(FitterResult):
         self.post_fit = cube_post_fit
         self.fg_med = fg_med
         self.eor = eor
+        self.noise = cube_noise
 
         FitterResult.__init__(self, cube_fit, cube_sub)
 
@@ -107,7 +108,7 @@ class GprForegroundResult(FitterResult):
         if fitutil.get_kern_startwith(self.gpr_res.model.kern, 'noise'):
             return self.get_model_component('noise', True)
 
-        n_freqs, n_modes = self.fit.data.shape
+        n_freqs, _ = self.fit.data.shape
         cov_err = np.zeros((n_freqs, n_freqs))
         cov_err[np.diag_indices(n_freqs)] = fitutil.get_model_noise(self.gpr_res)
         cov_err = datacube.ErrorCovariance(self.fit.freqs, cov_err, self.gpr_res.y_scale)
@@ -154,35 +155,8 @@ class GprForegroundResult(FitterResult):
 
     def test_single_parameter(self, param_name, min_value, max_value, n_optimize_loop=2,
                               tol=1e-3, verbose=False, method='bounded', brute_ns=20):
-        model = self.gpr_res.model.copy()
-        p = fitutil.get_model_parameter(model, param_name)
-
-        def func(v):
-            if v < 0:
-                return 1e99
-
-            p.constrain_fixed(v)
-            model.optimize_restarts(n_optimize_loop, verbose=False)
-
-            if verbose:
-                print('Test:', v, - model.log_likelihood() - model.log_prior())
-
-            return - model.log_likelihood() - model.log_prior()
-
-        def brute_finish(fct, pos, args):
-            return fmin(fct, pos, xtol=tol, ftol=tol, maxiter=brute_ns, full_output=1, disp=0)
-
-        if method == 'brute':
-            res = brute(func, [(min_value, max_value)], Ns=brute_ns, finish=brute_finish)
-            x0 = res[0]
-        elif method == 'bounded':
-            res = minimize_scalar(func, tol=tol, bounds=[min_value, max_value], method='bounded')
-            x0 = res.x
-        else:
-            res = minimize_scalar(func, bracket=[min_value, max_value], tol=tol, method=method)
-            x0 = res.x
-
-        return x0
+        return self.gpr_res.test_single_parameter(param_name, min_value, max_value, n_optimize_loop=n_optimize_loop,
+                                                  tol=tol, verbose=verbose, method=method, brute_ns=brute_ns)
 
     @staticmethod
     def load(dir_path, name):
@@ -203,6 +177,11 @@ class GprForegroundResult(FitterResult):
         if os.path.isfile(int_fg_full_file):
             int_fg_full = datacube.CartDataCube.load(os.path.join(dir_path, name + '.int_fg_full.h5'))
 
+        noise = None
+        noise_file = os.path.join(dir_path, name + '.noise.h5')
+        if os.path.isfile(noise_file):
+            noise = datacube.CartDataCube.load(os.path.join(dir_path, name + '.noise.h5'))
+
         kern = None
         kern_file = os.path.join(dir_path, name + '.kern.json')
         if os.path.isfile(kern_file):
@@ -215,9 +194,9 @@ class GprForegroundResult(FitterResult):
                 kern = None
 
         return GprForegroundResult(None, fit, sub, pre_fit, post_fit, fg_med=fg_med, eor=eor,
-                                   fg_med_full=fg_med_full, int_fg_full=int_fg_full, kern=kern)
+                                   fg_med_full=fg_med_full, int_fg_full=int_fg_full, kern=kern, cube_noise=noise)
 
-    def save(self, dir_path, name, save_no_gaps=True):
+    def save(self, dir_path, name):
         if not os.path.exists(dir_path):
             os.makedirs(dir_path)
         self.fit.save(os.path.join(dir_path, name + '.fit.h5'))
@@ -227,18 +206,21 @@ class GprForegroundResult(FitterResult):
         self.pre_fit.save(os.path.join(dir_path, name + '.pre_fit.h5'))
         self.post_fit.save(os.path.join(dir_path, name + '.post_fit.h5'))
 
-        self.fg_med_full.save(os.path.join(dir_path, name + '.fg_med_full.h5'))
+        if self.fg_med_full:
+            self.fg_med_full.save(os.path.join(dir_path, name + '.fg_med_full.h5'))
 
         if self.int_fg_full:
             self.int_fg_full.save(os.path.join(dir_path, name + '.int_fg_full.h5'))
 
+        if self.noise:
+            self.noise.save(os.path.join(dir_path, name + '.noise.h5'))
+
         try:
             with open(os.path.join(dir_path, name + '.kern.json'), mode='w', encoding='utf-8') as f:
                 f.write(json.dumps(self.kern.to_dict()))
         except NotImplementedError:
             # Some kernel do not support yet serialization. Ignore the error.
             print('Saving kernel parameter failed')
-            pass
 
 
 class MixForegroundResult(FitterResult):
@@ -297,18 +279,69 @@ class GprForegroundFit(AbstractForegroundFitter):
 
         cube_sub.set_weights(data_cube.weights)
 
-        return GprForegroundResult(gpr_res, cube_fit, cube_sub, cube_pre_fit, cube_post_fit)
+        return GprForegroundResult(gpr_res, cube_fit, cube_sub, cube_pre_fit, cube_post_fit, 
+                                   cube_noise=data_cube_noise)
+
+
+class ScaleNoiseGprForegroundFit(GprForegroundFit):
+
+    def __init__(self, gpr_config):
+        GprForegroundFit.__init__(self, gpr_config)
+
+    def run(self, data_cube, data_cube_noise, rnd_seed=None):
+        t = time.time()
+        np.random.seed(rnd_seed)
+
+        if self.config.use_simulated_noise:
+            sefd = data_cube_noise.new_with_cov_err().estimate_sefd()
+            print('Using simulated noise with sefd = %.1f Jy' % sefd)
+            weights = data_cube_noise.weights.copy()
+            if self.config.simulated_noise_scale_kper:
+                weights.scale_with_noise_cube(data_cube_noise, sefd_poly_fit_deg=2,
+                                              expected_sefd=sefd, scale_freqs=False)
+
+            noise_i = weights.simulate_noise(sefd, data_cube_noise.meta.total_time,
+                                             fake_apply_win_fct=True, hermitian=False)
+            noise_i.weights = data_cube_noise.weights.copy()
+            data_cube_noise = noise_i
+
+        do_scaled_noise = not self.config.fixed_noise and self.config.heteroscedastic
+
+        if do_scaled_noise:
+            self.config.fixed_noise_scale = 1
+            gpr_res = GprForegroundFit.run(self, data_cube, data_cube_noise, rnd_seed=rnd_seed)
+
+            n_optimize_loop = self.config.noise_scale_search_n_restarts
+            tol = self.config.noise_scale_search_tol
+            rmin, rmax = self.config.noise_scale_search_range
+            method = self.config.noise_scale_search_method
+            maxiter = self.config.noise_scale_search_maxiter
+
+            t = time.time()
+            print('Testing noise-scale value with method %s...\n' % method)
+
+            noise_scale = gpr_res.test_single_parameter('het_Gauss.variance', rmin, rmax, tol=tol,
+                                                        n_optimize_loop=n_optimize_loop,
+                                                        method=method, brute_ns=maxiter, verbose=True)
+
+            print('New noise_scale:', noise_scale)
+            data_cube_noise = np.sqrt(noise_scale) * data_cube_noise
+
+            print("\nDone in %.2f s\n" % (time.time() - t))
+
+        return GprForegroundFit.run(self, data_cube, data_cube_noise, rnd_seed=rnd_seed)
 
 
 class MultiGprForegroundResult(object):
 
-    def __init__(self, cube):
+    def __init__(self, cube, cube_noise):
         self.gpr_results = []
         self.data_idx = []
         self.fit = cube.new_with_data(np.zeros_like(cube.data))
         self.sub = cube.new_with_data(np.zeros_like(cube.data))
         self.pre_fit = cube.new_with_data(np.zeros_like(cube.data))
         self.post_fit = cube.new_with_data(np.zeros_like(cube.data))
+        self.cube_noise = cube_noise
 
     def add_res(self, gpr_res, data_idx):
         self.gpr_results.append(gpr_res)
@@ -338,7 +371,7 @@ class MultiBaselinesGprForegroundFit(GprForegroundFit):
         return datacube.CartDataCube(cube.data[:, idx], cube.uu[idx], cube.vv[idx], cube.freqs, cube.meta)
 
     def run(self, data_cube, data_cube_noise):
-        multi_gpr_res = MultiGprForegroundResult(data_cube)
+        multi_gpr_res = MultiGprForegroundResult(data_cube, data_cube_noise)
 
         for bs, be in psutil.pairwise(np.arange(data_cube.ru.min(), data_cube.ru.max() + self.du, self.du)):
             idx = (data_cube.ru >= bs) & (data_cube.ru <= be)
