@@ -242,6 +242,9 @@ class PowerSpectraConfig(psutil.SimpleConfig):
         self.add('rmean_axis', None, int)
 
         self._el = el
+        self.set('ps2d_pos_only', ps2d_pos_only)
+        self.set('ft_method', ft_method)
+        self.set('window_fct', window_fct)
 
     @property
     def el(self):
@@ -269,19 +272,28 @@ class PowerSpectraConfig(psutil.SimpleConfig):
     def copy(self):
         new = PowerSpectraConfig()
         new.config = self.config.copy()
-        new._el = self._el.copy()
+        if self._el is not None:
+            new._el = self._el.copy()
 
         return new
 
 
 class PowerSpectraBuilder(object):
 
-    def __init__(self, ps_config, eor_bin_list=None):
-        self.ps_config = PowerSpectraConfig.load(ps_config)
-        if eor_bin_list is not None:
-            self.eor_bin_list = EorBinList.load(eor_bin_list)
-        else:
-            self.eor_bin_list = None
+    def __init__(self, ps_config=None, eor_bin_list=None):
+        if ps_config is None:
+            ps_config = PowerSpectraConfig()
+        elif not isinstance(ps_config, PowerSpectraConfig):
+            ps_config = PowerSpectraConfig.load(ps_config)
+        
+        if eor_bin_list is None:
+            eor_bin_list = EorBinList()
+            eor_bin_list.add_freq('0', 10, 200)
+        elif not isinstance(eor_bin_list, EorBinList):
+            eor_bin_list = EorBinList.load(eor_bin_list)
+
+        self.ps_config = ps_config
+        self.eor_bin_list = eor_bin_list
 
     def get(self, cube, eor_bin_name=None, z=None, fmhz_range=None, **kargs):
         if z is not None:
@@ -1166,11 +1178,11 @@ class SpatialPowerSpectra(PowerSpectraMath):
             fig, ax = plt.subplots()
 
         if weighted and isinstance(self.w, np.ndarray):
-            y = np.average(self.data, weights=self.w, axis=0)
-            y_err = np.sqrt(np.sum((self.err * self.w) ** 2, axis=0)) / self.w.sum(axis=0)
+            y = psutil.nanaverage(self.data, self.w, axis=0)
+            y_err = np.sqrt(np.nansum((self.err * self.w) ** 2, axis=0)) / np.nansum(self.w, axis=0)
         else:
-            y = np.mean(self.data, axis=0)
-            y_err = np.sqrt(np.sum(self.err ** 2, axis=0)) / len(self.freqs)
+            y = np.nanmean(self.data, axis=0)
+            y_err = np.sqrt(np.nansum(self.err ** 2, axis=0)) / len(self.freqs)
 
         y, y_err, temp_unit = self.temp_conversion(y, y_err, mkelvin=mkelvin, kelvin_square=kelvin_square)
 
